@@ -27,15 +27,28 @@ from hashlib import sha1
 import hmac
 import logging
 from email.utils import formatdate
+from botocore.exceptions import UnknownSignatureVersionError
+from botocore.utils import normalize_url_path
+from botocore.compat import HTTPHeaders
+from six.moves import http_client
 
 logger = logging.getLogger(__name__)
 
 try:
     from urllib.parse import quote
+    from urllib.parse import unquote
     from urllib.parse import urlsplit
+    from urllib.parse import parse_qsl
 except ImportError:
     from urllib import quote
+    from urllib import unquote
     from urlparse import urlsplit
+    from urlparse import parse_qsl
+
+
+PostContentType = 'application/x-www-form-urlencoded; charset=UTF-8'
+EMPTY_SHA256_HASH = (
+    'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')
 
 
 class SigV2Auth(object):
@@ -47,23 +60,19 @@ class SigV2Auth(object):
         self.service_name = service_name
         self.region_name = region_name
 
-    def calc_signature(self, args):
-        split = urlsplit(args['url'])
+    def calc_signature(self, request, params):
+        split = urlsplit(request.url)
         path = split.path
         if len(path) == 0:
             path = '/'
-        string_to_sign = '%s\n%s\n%s\n' % (args['method'],
+        string_to_sign = '%s\n%s\n%s\n' % (request.method,
                                            split.netloc,
                                            path)
         lhmac = hmac.new(self.credentials.secret_key.encode('utf-8'),
-                        digestmod=sha256)
-        args['params']['SignatureMethod'] = 'HmacSHA256'
-        if self.credentials.token:
-            args['params']['SecurityToken'] = self.credentials.token
-        sorted_params = sorted(args['params'])
+                         digestmod=sha256)
         pairs = []
-        for key in sorted_params:
-            value = args['params'][key]
+        for key in sorted(params):
+            value = params[key]
             pairs.append(quote(key, safe='') + '=' +
                          quote(value, safe='-_~'))
         qs = '&'.join(pairs)
@@ -74,18 +83,48 @@ class SigV2Auth(object):
         b64 = base64.b64encode(lhmac.digest()).strip().decode('utf-8')
         return (qs, b64)
 
-    def add_auth(self, args):
-        args['params']['AWSAccessKeyId'] = self.credentials.access_key
-        args['params']['SignatureVersion'] = '2'
-        args['params']['Timestamp'] = datetime.datetime.utcnow().isoformat()
-        qs, signature = self.calc_signature(args)
-        args['params']['Signature'] = signature
-        if args['method'] == 'POST':
-            args['data'] = args['params']
-            args['params'] = {}
+    def add_auth(self, request):
+        # The auth handler is the last thing called in the
+        # preparation phase of a prepared request.
+        # Because of this we have to parse the query params
+        # from the request body so we can update them with
+        # the sigv2 auth params.
+        if request.data:
+            # POST
+            params = request.data
+        else:
+            # GET
+            params = request.param
+        params['AWSAccessKeyId'] = self.credentials.access_key
+        params['SignatureVersion'] = '2'
+        params['SignatureMethod'] = 'HmacSHA256'
+        params['Timestamp'] = datetime.datetime.utcnow().isoformat()
+        if self.credentials.token:
+            params['SecurityToken'] = self.credentials.token
+        qs, signature = self.calc_signature(request, params)
+        params['Signature'] = signature
+        return request
 
 
-PostContentType = 'application/x-www-form-urlencoded; charset=UTF-8'
+class SigV3Auth(object):
+    def __init__(self, credentials, service_name=None, region_name=None):
+        self.credentials = credentials
+        self.service_name = service_name
+        self.region_name = region_name
+
+    def add_auth(self, request):
+        if 'Date' not in request.headers:
+            request.headers['Date'] = formatdate(usegmt=True)
+        if self.credentials.token:
+            request.headers['X-Amz-Security-Token'] = self.credentials.token
+        new_hmac = hmac.new(self.credentials.secret_key.encode('utf-8'),
+                            digestmod=sha256)
+        new_hmac.update(request.headers['Date'].encode('utf-8'))
+        encoded_signature = base64.encodestring(new_hmac.digest()).strip()
+        signature = ('AWS3-HTTPS AWSAccessKeyId=%s,Algorithm=%s,Signature=%s' %
+                     (self.credentials.access_key, 'HmacSHA256',
+                      encoded_signature.decode('utf-8')))
+        request.headers['X-Amzn-Authorization'] = signature
 
 
 class SigV4Auth(object):
@@ -107,45 +146,27 @@ class SigV4Auth(object):
             sig = hmac.new(key, msg.encode('utf-8'), sha256).digest()
         return sig
 
-    def headers_to_sign(self, args):
+    def headers_to_sign(self, request):
         """
         Select the headers from the request that need to be included
         in the StringToSign.
         """
-        headers_to_sign = {}
-        headers_to_sign = {'Host': self.split.netloc}
-        for name, value in args['headers'].items():
+        header_map = HTTPHeaders()
+        split = urlsplit(request.url)
+        for name, value in request.headers.items():
             lname = name.lower()
-            if lname.startswith('x-amz'):
-                headers_to_sign[name] = value
-            elif lname == 'content-type':
-                headers_to_sign[name] = value
-        return headers_to_sign
+            header_map[lname] = value
+        if 'host' not in header_map:
+            header_map['host'] = split.netloc
+        return header_map
 
-    def build_payload(self, args):
-        """
-        For all Query-style requests, we will use POST.
-
-        When using SigV4, we need to convert the query parameters
-        to a string and place that in the payload of the POST request.
-        """
-        if args['method'] == 'POST' and not args['data']:
-            parameter_names = sorted(args['params'].keys())
-            pairs = []
-            for pname in parameter_names:
-                pval = str(args['params'][pname]).encode('utf-8')
-                pairs.append(quote(pname, safe='') + '=' +
-                             quote(pval, safe='-_~'))
-            args['data'] = '&'.join(pairs)
-            args['params'] = {}
-            args['headers']['content-type'] = PostContentType
-
-    def canonical_query_string(self, args):
+    def canonical_query_string(self, request):
         cqs = ''
-        if args['method'] == 'GET':
+        if request.params:
+            params = request.params
             l = []
-            for param in args['params']:
-                value = str(args['params'][param])
+            for param in params:
+                value = str(params[param])
                 l.append('%s=%s' % (quote(param, safe='-_.~'),
                                     quote(value, safe='-_.~')))
             l = sorted(l)
@@ -159,33 +180,34 @@ class SigV4Auth(object):
         case, sorting them in alphabetical order and then joining
         them into a string, separated by newlines.
         """
-        l = ['%s:%s' % (n.lower().strip(),
-                      headers_to_sign[n].strip()) for n in headers_to_sign]
-        l = sorted(l)
-        return '\n'.join(l)
+        headers = []
+        for key in set(headers_to_sign):
+            value = ','.join(v.strip() for v in
+                            sorted(headers_to_sign.get_all(key)))
+            headers.append('%s:%s' % (key, value))
+        headers.sort()
+        return '\n'.join(headers)
 
     def signed_headers(self, headers_to_sign):
-        l = ['%s' % n.lower().strip() for n in headers_to_sign]
+        l = ['%s' % n.lower().strip() for n in set(headers_to_sign)]
         l = sorted(l)
         return ';'.join(l)
 
-    def payload(self, args):
-        if args['data']:
-            return sha256(args['data']).hexdigest()
+    def payload(self, request):
+        if request.body:
+            return sha256(request.body.encode('utf-8')).hexdigest()
         else:
-            return sha256('').hexdigest()
+            return EMPTY_SHA256_HASH
 
-    def canonical_request(self, args):
-        cr = [args['method'].upper()]
-        path = self.split.path
-        if len(path) == 0:
-            path = '/'
+    def canonical_request(self, request):
+        cr = [request.method.upper()]
+        path = normalize_url_path(urlsplit(request.url).path)
         cr.append(path)
-        cr.append(self.canonical_query_string(args))
-        headers_to_sign = self.headers_to_sign(args)
+        cr.append(self.canonical_query_string(request))
+        headers_to_sign = self.headers_to_sign(request)
         cr.append(self.canonical_headers(headers_to_sign) + '\n')
         cr.append(self.signed_headers(headers_to_sign))
-        cr.append(self.payload(args))
+        cr.append(self.payload(request))
         return '\n'.join(cr)
 
     def scope(self, args):
@@ -204,19 +226,19 @@ class SigV4Auth(object):
         scope.append('aws4_request')
         return '/'.join(scope)
 
-    def string_to_sign(self, args, canonical_request):
+    def string_to_sign(self, request, canonical_request):
         """
         Return the canonical StringToSign as well as a dict
         containing the original version of all headers that
         were included in the StringToSign.
         """
         sts = ['AWS4-HMAC-SHA256']
-        sts.append(args['headers']['X-Amz-Date'])
-        sts.append(self.credential_scope(args))
-        sts.append(sha256(canonical_request).hexdigest())
+        sts.append(self.timestamp)
+        sts.append(self.credential_scope(request))
+        sts.append(sha256(canonical_request.encode('utf-8')).hexdigest())
         return '\n'.join(sts)
 
-    def signature(self, args, string_to_sign):
+    def signature(self, string_to_sign):
         key = self.credentials.secret_key
         k_date = self._sign(('AWS4' + key).encode('utf-8'),
                             self.timestamp[0:8])
@@ -225,27 +247,28 @@ class SigV4Auth(object):
         k_signing = self._sign(k_service, 'aws4_request')
         return self._sign(k_signing, string_to_sign, hex=True)
 
-    def add_auth(self, args):
+    def add_auth(self, request):
         # This could be a retry.  Make sure the previous
         # authorization header is removed first.
-        self.split = urlsplit(args['url'])
-        self.build_payload(args)
-        if 'X-Amzn-Authorization' in args['headers']:
-            del args['headers']['X-Amzn-Authorization']
-        args['headers']['X-Amz-Date'] = self.timestamp
+        split = urlsplit(request.url)
+        if 'Authorization' in request.headers:
+            del request.headers['Authorization']
+        if 'Date' not in request.headers:
+            request.headers['X-Amz-Date'] = self.timestamp
         if self.credentials.token:
-            args['headers']['X-Amz-Security-Token'] = self.credentials.token
-        canonical_request = self.canonical_request(args)
+            request.headers['X-Amz-Security-Token'] = self.credentials.token
+        canonical_request = self.canonical_request(request)
         logger.debug('CanonicalRequest:\n%s' % canonical_request)
-        string_to_sign = self.string_to_sign(args, canonical_request)
+        string_to_sign = self.string_to_sign(request, canonical_request)
         logger.debug('StringToSign:\n%s' % string_to_sign)
-        signature = self.signature(args, string_to_sign)
+        signature = self.signature(string_to_sign)
         logger.debug('Signature:\n%s' % signature)
-        headers_to_sign = self.headers_to_sign(args)
-        l = ['AWS4-HMAC-SHA256 Credential=%s' % self.scope(args)]
+        l = ['AWS4-HMAC-SHA256 Credential=%s' % self.scope(request)]
+        headers_to_sign = self.headers_to_sign(request)
         l.append('SignedHeaders=%s' % self.signed_headers(headers_to_sign))
         l.append('Signature=%s' % signature)
-        args['headers']['Authorization'] = ','.join(l)
+        request.headers['Authorization'] = ', '.join(l)
+        return request
 
 
 class HmacV1Auth(object):
@@ -269,7 +292,7 @@ class HmacV1Auth(object):
         new_hmac = hmac.new(self.credentials.secret_key.encode('utf-8'),
                             digestmod=sha1)
         new_hmac.update(string_to_sign.encode('utf-8'))
-        return base64.encodestring(new_hmac.digest()).strip()
+        return base64.encodestring(new_hmac.digest()).strip().decode('utf-8')
 
     def canonical_standard_headers(self, headers):
         interesting_headers = ['content-md5', 'content-type', 'date']
@@ -294,24 +317,32 @@ class HmacV1Auth(object):
             if headers[key] != None:
                 # TODO: move hardcoded prefix to provider
                 if lk.startswith('x-amz-'):
-                    custom_headers[lk] = headers[key].strip()
+                    custom_headers[lk] = ','.join(v.strip() for v in
+                                                   headers.get_all(key))
         sorted_header_keys = sorted(custom_headers.keys())
         hoi = []
         for key in sorted_header_keys:
             hoi.append("%s:%s" % (key, custom_headers[key]))
         return '\n'.join(hoi)
 
-    def canonical_resource(self, path):
+    def unquote_v(self, nv):
+        """
+        TODO: Do we need this?
+        """
+        if len(nv) == 1:
+            return nv
+        else:
+            return (nv[0], unquote(nv[1]))
+
+    def canonical_resource(self, split):
         # don't include anything after the first ? in the resource...
         # unless it is one of the QSA of interest, defined above
-        buf = ''
-        t = path.split('?')
-        buf += t[0]
+        buf = split.path
 
-        if len(t) > 1:
-            qsa = t[1].split('&')
+        if split.query:
+            qsa = split.query.split('&')
             qsa = [a.split('=', 1) for a in qsa]
-            qsa = [unquote_v(a) for a in qsa if a[0] in self.QSAOfInterest]
+            qsa = [self.unquote_v(a) for a in qsa if a[0] in self.QSAOfInterest]
             if len(qsa) > 0:
                 qsa.sort(cmp=lambda x, y:cmp(x[0], y[0]))
                 qsa = ['='.join(a) for a in qsa]
@@ -320,39 +351,40 @@ class HmacV1Auth(object):
 
         return buf
 
-    def canonical_string(self, method, path, headers, expires=None):
+    def canonical_string(self, method, split, headers, expires=None):
         cs = method.upper() + '\n'
         cs += self.canonical_standard_headers(headers) + '\n'
         custom_headers = self.canonical_custom_headers(headers)
         if custom_headers:
             cs += custom_headers + '\n'
-        cs += self.canonical_resource(path)
+        cs += self.canonical_resource(split)
         return cs
 
-    def get_signature(self, method, path, headers, expires=None):
+    def get_signature(self, method, split, headers, expires=None):
         if self.credentials.token:
             #TODO: remove hardcoded header name
             headers['security_token'] = self.credentials.token
         string_to_sign = self.canonical_string(method,
-                                               path,
+                                               split,
                                                headers)
         logger.debug('StringToSign:\n%s' % string_to_sign)
         return self.sign_string(string_to_sign)
 
-    def add_auth(self, args):
-        split = urlsplit(args['url'])
-        path = split.path
-        signature = self.get_signature(args['method'], path,
-                                       args['headers'])
-        args['headers']['Authorization'] = ("AWS %s:%s" % (self.credentials.access_key,
+    def add_auth(self, request):
+        split = urlsplit(request.url)
+        signature = self.get_signature(request.method, split,
+                                       request.headers)
+        request.headers['Authorization'] = ("AWS %s:%s" % (self.credentials.access_key,
                                                            signature))
 
 
 def get_auth(signature_version, *args, **kw):
     if signature_version == 'v2':
         return SigV2Auth(*args, **kw)
-    if signature_version == 'v4':
+    elif signature_version == 'v4':
         return SigV4Auth(*args, **kw)
-    if signature_version == 's3':
+    elif signature_version == 's3':
         return HmacV1Auth(*args, **kw)
-    raise ValueError('Unknown auth scheme: %s' % auth_name)
+    elif signature_version == 'v3https':
+        return SigV3Auth(*args, **kw)
+    raise UnknownSignatureVersionError(signature_version=signature_version)
