@@ -22,6 +22,7 @@
 #
 import logging
 from .parameters import get_parameter
+from .exceptions import MissingParametersError
 from . import BotoCoreObject
 
 logger = logging.getLogger(__name__)
@@ -52,8 +53,6 @@ class Operation(BotoCoreObject):
         self.params = []
         if self.input and 'members' in self.input:
             for name, data in self.input['members'].items():
-                if self.service:
-                    data['flattened'] = not self.service.membered_lists
                 param = get_parameter(name, data)
                 self.params.append(param)
 
@@ -84,7 +83,6 @@ class Operation(BotoCoreObject):
                                       kwargs[param.py_name],
                                       built_params)
         if missing:
-            msg = 'The following required parameters are missing:'
-            msg += ','.join([p.py_name for p in missing])
-            raise ValueError(msg)
+            missing_str = ','.join([p.py_name for p in missing])
+            raise MissingParametersError(missing=missing_str)
         return built_params
